<?php

namespace Drupal\commerce_paykeeper\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;

require_once 'help.class.php';
/**
 * Provides the class for payment off-site form.
 *
 * Provide a buildConfigurationForm() method which calls buildRedirectForm()
 * with the right parameters.
 */
class PaykeeperpaymentForm extends BasePaymentOffsiteForm {

  /**
   * Gateway plugin.
   *
   * @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface
   */
  private $paymentGatewayPlugin;

  /**
   * Getting plugin's configuration.
   *
   * @param string $configuration
   *   Configuration name.
   *
   * @return mixed
   *   Configuration value.
   */
  private function getConfiguration($configuration) {
    return $this->paymentGatewayPlugin->getConfiguration()[$configuration];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment                = $this->entity;

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    // Получение настроек
    $configuration          = $payment_gateway_plugin->getConfiguration();
    $redirect_url           = $configuration["payment_url"];
    $order                  = $payment->getOrder();
    $billing_address        = $order->getBillingProfile()->get('address');
    $payment_address        = $billing_address->getEntity()->getFields()['address']->getValue()[0];
    $service_name           = 'paykeeper_payment_system';
    $pk_obj = new \HelpClass();
    $pk_obj->setOrderParams
    (
    //sum
      $order->getTotalPrice() ? number_format($order->getTotalPrice()->getNumber(),2,'.','') : 0,
      //clientid
      $payment_address["given_name"] .' '. $payment_address["family_name"] ?? $order->getCustomer()->getDisplayName()??$order->getCustomer()->getAccountName(),
      //orderid
      $order->id(),
      //client_email
      $order->getEmail() ?? 'no_email',
      //client_phone
      '',
      //service_name
      'paykeeper_payment_system',
      //payment form url
      $configuration['payment_url'],
      //secret key
      $configuration["secret_word"]
    );

    $cart = [];
    $adjustments = $order->collectAdjustments();
    $tax_arr=[];
    $shippping_arr=[];
    $shipping_it=[];
    foreach( $adjustments as $item)
    {

      if($item->getType()=='tax')
      {
        $tax_it['amount']=number_format($item->getAmount()->getNumber(),2,'.','');
        $tax_it['percentage']=number_format($item->getPercentage()*100,0,'.','');
        $tax_it['included']=$item->isIncluded();
        $tax_arr[]=$tax_it;
      }
      if ($item->getType()=='shipping')
      {
        $shipping_it['label']=$item->getLabel();
        $shipping_it['amount']=$item->getAmount()->getNumber();
        $shippping_arr[]=$shipping_it;
      }
    }
    // Собираем корзину
    $cart_index = 0;
    $tax_amount = 0;
    foreach($order->getItems() as $OrderItem)
    {
      $quantity =  $OrderItem->getQuantity();
      //Если налог не включается в стоимость товара при отображении, то $tax_amount>0, если включается то $tax_amount=0

        if (count($tax_arr)>0)
      {
          $tax_amount = !$tax_arr[$cart_index]['included']?$tax_arr[$cart_index]['amount']/$quantity:0;
      }

      //Если  налог не включается в стоимость товара при отображении, то $full_tax_amount>0, если нет =0
      //$full_tax_amount = !$tax_arr[$cart_index]['included']?$tax_arr[$cart_index]['amount']:0;
      $price = number_format($OrderItem->getUnitPrice()->getNumber()+$tax_amount,2,'.','');
      $item_sum = number_format($price*$quantity,2,'.','');
      $vat = count($tax_arr)>0?'vat'.$tax_arr[$cart_index]['percentage']:'none';
      $pk_obj->use_taxes=isset($tax_arr[$cart_index]['percentage'])?true:false;
      if ($quantity == 1 && $pk_obj->single_item_index < 0)
        $pk_obj->single_item_index = $cart_index;
      if ($quantity > 1 && $pk_obj->more_then_one_item_index < 0)
        $pk_obj->more_then_one_item_index = $cart_index;
      $pk_obj->updateFiscalCart
      (
        $pk_obj->getPaymentFormType(),
        $OrderItem->getTitle(),
        $price,
        $quantity,
        $item_sum,
        $vat,
      );
      $cart_index++;
    }
    // Добавляем доставку в корзину
    if (count($shippping_arr)>0)
    {
      $pk_obj->use_delivery=true;
      $pk_obj->setShippingPrice(number_format($shippping_arr[0]['amount'],2,'.',''));
      $pk_obj->delivery_index = count($pk_obj->getFiscalCart());
      $pk_obj->updateFiscalCart
      (
        $pk_obj->getPaymentFormType(),
        $shippping_arr[0]['label'],
        $pk_obj->getShippingPrice(),
        1,
        $pk_obj->getShippingPrice(),
        'none'
      );

    }
    $pk_obj->setDiscounts($configuration['forced_discounts_check']==1);
    $pk_obj->correctPrecision();
    if ($pk_obj->getPaymentFormType() == "create") {
      $to_hash = number_format($pk_obj->getOrderTotal(), 2, ".", "") .
        $pk_obj->getOrderParams("clientid") .
        $pk_obj->getOrderParams("orderid") .
        $pk_obj->getOrderParams("service_name") .
        $pk_obj->getOrderParams("client_email") .
        $pk_obj->getOrderParams("client_phone") .
        $pk_obj->getOrderParams("secret_key");
    };
    $sign = hash ('sha256' , $to_hash);

    $data =
      [
        'sum' => $pk_obj->getOrderTotal(),
        'orderid' => $pk_obj->getOrderParams('orderid'),
        'clientid' => $pk_obj->getOrderParams('clientid'),
        'client_email' => $pk_obj->getOrderParams('client_email'),
        'client_phone' => $pk_obj->getOrderParams('client_phone'),
        'service_name' => $service_name,
        'cart' =>  json_encode($pk_obj->getFiscalCart()),
        'sign' => $sign
      ];
    return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, 'post') ;

  }

}
